#include <Arduino_LSM6DS3.h>
#include <SPI.h>
#include <ArduinoBLE.h>

#define NUM_SAMPLES 10

float baselinePitch = 0;
float filterArray[NUM_SAMPLES];
int sampleIndex = 0;
int dorsiflexionCount = 0;
int plantarflexionCount = 0;
int previousPitch = 0;

const char* UUID_serv = "84582cd0-3df0-4e73-9496-29010d7445dd";
const char* UUID_pitch = "84582cd1-3df0-4e73-9496-29010d7445dd";
const char* UUID_dorsiflexion = "84582cd2-3df0-4e73-9496-29010d7445dd";
const char* UUID_plantarflexion = "84582cd3-3df0-4e73-9496-29010d7445dd";

BLEService myService(UUID_serv);
BLEFloatCharacteristic chAX(UUID_pitch, BLERead | BLENotify);
BLEFloatCharacteristic chDorsiflexion(UUID_dorsiflexion, BLERead | BLENotify);
BLEFloatCharacteristic chPlantarFlexion(UUID_plantarflexion, BLERead | BLENotify);

void setup() {
  if (!IMU.begin()) {
    while (1); // halt the program if IMU initialization fails
  }

  for (int i = 0; i < 100; i++) {
    if (IMU.accelerationAvailable()) {
      float x, y, z;
      IMU.readAcceleration(x, y, z);
      float pitch = atan2(-x, sqrt(y * y + z * z)) * 180.0 / PI;
      baselinePitch += pitch;
      delay(10);
    }
  }
  baselinePitch /= 100;

  if (!BLE.begin()) {
    while (1); // halt the program if BLE initialization fails
  }

  BLE.setLocalName("Nano33IOT_IMU");
  BLE.setAdvertisedService(myService);
  myService.addCharacteristic(chAX);
  myService.addCharacteristic(chDorsiflexion);
  myService.addCharacteristic(chPlantarFlexion);
  BLE.addService(myService);

  chAX.writeValue(0.0);
  chDorsiflexion.writeValue(0.0);
  chPlantarFlexion.writeValue(0.0);
  BLE.advertise();
}

void loop() {
  BLEDevice central = BLE.central();

  if (central) {
    while (central.connected()) {
      float x, y, z;

      if (IMU.accelerationAvailable()) {
        IMU.readAcceleration(x, y, z);

        float pitch = atan2(-x, sqrt(y * y + z * z)) * 180.0 / PI - baselinePitch;
        filterArray[sampleIndex % NUM_SAMPLES] = pitch;
        float filteredPitch = 0;
        for (int i = 0; i < NUM_SAMPLES; i++) {
          filteredPitch += filterArray[i];
        }
        filteredPitch /= NUM_SAMPLES;
        sampleIndex++;

        int intPitch = (int)round(filteredPitch); // Convert to integer

        if (intPitch != previousPitch) {
          if (previousPitch == 0 && intPitch > 0) {
            dorsiflexionCount++;
            chDorsiflexion.writeValue(dorsiflexionCount);
          } else if (previousPitch == 0 && intPitch < 0) {
            plantarflexionCount++;
            chPlantarFlexion.writeValue(plantarflexionCount);
          }
        }

        chAX.writeValue((float)intPitch); // Write integer value as float
        previousPitch = intPitch;

        delay(100);
      }
    }
  }
}
